import re
import shutil
from pathlib import Path


def rename_document(path: str, new_suffix) -> Path:
    """Изменение расширения файла."""
    doc_name = Path(path).parent / f'{Path(path).name.removesuffix(Path(path).suffix)}{new_suffix}'
    Path(path).rename(doc_name)
    return doc_name


def extract_doc(path: Path) -> Path:
    """Распаковка архива."""
    path_unpack = path.parent / Path(path).name.removesuffix(Path(path).suffix)
    shutil.unpack_archive(path, path_unpack, "zip")
    return path_unpack


def delete_protection_docx(file: Path) -> None:
    """Удаление защиты из документа. Сохранение без удаленного элемента."""
    with open(file, "r", encoding="utf-8") as set_xml:
        txt_xml = set_xml.read()
        updated_xml_string = re.sub(r'<w:documentProtection.*?/>', '', txt_xml)
    with open(file, "w", encoding="utf-8") as set_xml:
        set_xml.write(updated_xml_string)


def delete_protection_odt(file: Path) -> None:
    """Удаление защиты из документа. Сохранение без удаленного элемента."""
    with open(file, "r", encoding="utf-8") as set_xml:
        txt_xml = set_xml.read()
        protect_sections = re.findall(r'<text:section.*?>', txt_xml)
        for protect_section in protect_sections:
            replace_section = " ".join([x for x in protect_section.split()
                                        if not x.startswith(("text:protected", "text:protection-key"))]) + ">"
            txt_xml = re.sub(protect_section, replace_section, txt_xml)
    with open(file, "w", encoding="utf-8") as set_xml:
        set_xml.write(txt_xml)


def delete_protection_xlsx(path: Path) -> None:
    """Удаление защиты из документа. Сохранение без удаленного элемента."""
    path_sheets = path / "xl" / "worksheets"
    files = [x for x in Path(path_sheets).iterdir()]
    for file in files:
        with open(file, "r", encoding="utf-8") as set_xml:
            txt_xml = set_xml.read()
            updated_xml_string = re.sub(r'<sheetProtection.*?/>', '', txt_xml)
        with open(file, "w", encoding="utf-8") as set_xml:
            set_xml.write(updated_xml_string)


def delete_protection_ods(file: Path) -> None:
    """Удаление защиты из документа. Сохранение без удаленного элемента."""
    with open(file, "r", encoding="utf-8") as set_xml:
        txt_xml = set_xml.read()
        txt_xml = re.sub(r'<office:spreadsheet.*?>', '<office:spreadsheet>', txt_xml)
        tables = re.findall(r'<table:table table:name.*?>', txt_xml)
        for table in tables:
            tag = [x for x in table.split() if not x.startswith(("table:protected", "table:protection-key"))]
            find_tag = " ".join(tag[0:2]) + ".*?>"
            remove_tag = " ".join(tag) + ">"
            txt_xml = re.sub(find_tag, remove_tag, txt_xml)
            txt_xml = re.sub(r'<loext:table-protection.*?/>', '', txt_xml)
    with open(file, "w", encoding="utf-8") as set_xml:
        set_xml.write(txt_xml)


def zip_document(doc: str, path: Path) -> None:
    """Упаковка директории в архив."""
    shutil.make_archive(str(path.parent / doc), 'zip', path)


def main() -> None:
    path_doc_file = input("Введите путь к документу: ")
    if not Path(path_doc_file).exists():
        print("Указанного файла не существует")
        exit()
    elif Path(path_doc_file).suffix not in [".docx", ".odt", ".xlsx", ".ods"]:
        print("Неверный тип файла")
        exit()
    origin_suffix = Path(path_doc_file).suffix  # оригинальное расширение файла
    rename_doc = rename_document(path_doc_file, ".zip")  # имя файла после смены расширения
    path_doc = extract_doc(rename_doc)  # директория куда распакован файл
    Path(rename_doc).unlink()  # удаление оригинального файла
    if origin_suffix == ".docx":
        delete_protection_docx(path_doc / "word" / "settings.xml")  # удаление защиты
    elif origin_suffix == ".odt":
        delete_protection_odt(path_doc / "content.xml")  # удаление защиты
    elif origin_suffix == ".xlsx":
        delete_protection_xlsx(path_doc)  # удаление защиты
    elif origin_suffix == ".ods":
        delete_protection_ods(path_doc / "content.xml")
    zip_document(f'{Path(rename_doc).name.removesuffix(Path(rename_doc).suffix)}',
                 Path(path_doc))  # упаковка файла в архив
    rename_document(str(rename_doc), origin_suffix)  # переименование архива в оригинальное имя
    shutil.rmtree(path_doc)  # удаление директории с распакованным документом


if __name__ == "__main__":
    main()
